@@ -1,7 +1,21 @@
 /*
- * $Id: direct.c,v 1.3 1994/06/21 08:14:11 ralf Exp $
+ * $Id: direct.c,v 1.7 1998/10/28 04:09:56 glehner Exp $
  * History:
  * $Log: direct.c,v $
+ * Revision 1.7  1998/10/28 04:09:56  glehner
+ * (WriteFontName): Cleaned up. Eliminated unecessary warning
+ * and not completed rtf-output when using *Font*.
+ *
+ * Revision 1.6  1998/07/03 07:03:16  glehner
+ * lclint cleaning
+ *
+ * Revision 1.5  1997/02/15 20:45:41  ralf
+ * Some lclint changes and corrected variable declarations
+ *
+ * Revision 1.4  1995/03/23 15:58:08  ralf
+ * Reworked version by Friedrich Polzer and Gerhard Trisko
+ *
+ *
  * Revision 1.3  1994/06/21  08:14:11  ralf
  * Corrected Bug in keyword search
  *
@@ -15,6 +29,8 @@
 /***************************************************************************
      name : direct.c
    author : DORNER Fernando, GRANZER Andreas
+            POLZER Friedrich,TRISKO Gerhard
+  * changed TryDirectConvert: use search on sorted array
   purpose : This file is used for converting LaTeX commands by simply text exchange
  ******************************************************************************/
 
@@ -24,11 +40,12 @@
 #include "main.h"
 #include "direct.h"
 #include "fonts.h"
+#include "cfg.h"
 /******************************************************************************/
 
-/*******************************  extern variables ****************************/
-extern char *progname;
-/******************************************************************************/
+/*************************** prototypes **************************************/
+
+static bool WriteFontName(const char **buffpoint, FILE *fRtf);
 
 /******************************* defines *************************************/
 #define MAXFONTLEN 100
@@ -36,18 +53,18 @@ extern char *progname;
 
 
 /******************************************************************************/
-BOOL WriteFontName(char **buffpoint, FILE *fRtf)
+bool WriteFontName(const char **buffpoint, FILE *fRtf)
 /******************************************************************************
-  purpose: reads from the file fonts.cfg how some easy LaTex-commands can be
-	   converted into Rtf-commands by text exchange
-	   especially font-number-type-convertions
+  purpose: reads from the font-array to write correct font-number into
+           Rtf-File
 parameter: buffpoint: font and number
 	   fRtf: File-Pointer to Rtf-File
+globals:   progname
  ******************************************************************************/
 {
   char buffer[MAXFONTLEN+1];
   int i;
-  int fnumber;
+  size_t fnumber;
 
   if (**buffpoint == '*')
   {
@@ -59,8 +76,9 @@ parameter: buffpoint: font and number
   {
     if ((i >= MAXFONTLEN) || (**buffpoint == '\0'))
     {
-      fprintf(stderr, "\n%s: ERROR: Invalid fontname in direct command",progname);
-      exit(-1);
+      fprintf(stderr, "\n%s: ERROR: Invalid fontname in direct command",
+	      progname);
+      exit(EXIT_FAILURE);
     }
     buffer[i] = **buffpoint;
     i++;
@@ -71,105 +89,74 @@ parameter: buffpoint: font and number
   {
     fprintf(stderr, "\n%s: ERROR: Unknown fontname in direct command",progname);
     fprintf(stderr, "\nprogram aborted\n");
-    exit(-1);
+    exit(EXIT_FAILURE);
   }
   else
   {
-    fprintf(fRtf,"%d",fnumber);
+    fprintf(fRtf,"%u",(unsigned int)fnumber);
     return TRUE;
   }
 }
 
 
-
-/******************************************************************************/
-BOOL TryDirectConvert(char *command, FILE *fRtf)
 /******************************************************************************
-  purpose: reads from the file direct.cfg how some easy LaTex-commands can be
+  purpose: reads from the direct-array how some easy LaTex-commands can be
 	   converted into Rtf-commands by text exchange
 parameter: command: LaTex-command and Rtf-command
 	   fRtf: File-Pointer to Rtf-File
+globals:   progname
  ******************************************************************************/
+bool
+TryDirectConvert(char *command, FILE *fRtf)
 {
-  FILE *fp;
-  char buffer[512];
-  int i;
-  char *buffpoint;
+  const char *buffpoint;
+  const char *RtfCommand;
+  char TexCommand[128];
 
   if (strlen(command) >= 100)
-      {
-      fprintf(stderr,"\n%s: WARNING: Command %s is too long in direct.cfg.\n",progname,command);
+    {
+      fprintf(stderr,"\n%s: WARNING: Command %s is too long in LaTeX-File.\n",progname,command);
       return FALSE;    /* command too long */
-      }
-  fp = open_cfg("direct.cfg");
-  for(;;)
-  {
-    if ( (fgets( buffer, 512, fp )) == NULL)
-    {
-      fclose(fp);
-      return FALSE;
     }
-    if (strlen(buffer) > 500)
+  
+  TexCommand[0] = '\\';
+  TexCommand[1] = '\0';
+  strcat (TexCommand, command);
+  
+  RtfCommand = SearchRtfCmd (TexCommand, DIRECT_A);
+  if (RtfCommand == NULL)
+    return FALSE;
+  
+  buffpoint = RtfCommand;
+  diagnostics(4, "Direct converting `%s' command to `%s'.",
+	      TexCommand, RtfCommand);
+  while (buffpoint[0] != '\0')
     {
-      fprintf(stderr, "\n%s: WARNING: line too long in DIRECT.CFG - only 500 characters\n",progname);
-      fclose(fp);
-      return FALSE;
-    }
-    buffpoint = buffer;
-    while (*buffpoint != '\\')
-    {
-      if ( (*buffpoint == '#') || (*buffpoint == '\0') )
-	break;
-      ++buffpoint;
-    }
-    if (*buffpoint != '\\')
-      continue;
-    ++buffpoint;
-    for ((int)i = 0; (int)i < (int)strlen(command); i++)
-    {
-      if (buffpoint[i] == '\0')
-	break;
-      if (buffpoint[i] != command[i])
-	break;
-    }
-    if ((int)i == (int)strlen(command) && buffpoint[i] == ',')   /* found */
-    {
-      buffpoint += i;
-      while (*buffpoint != ',')
-      {
-	if ( (*buffpoint == '#') || (*buffpoint == '\0') )
-	{
-	  fprintf(stderr, "\n%s: WARNING: error in direct command file - missing , \n",progname);
-	  fclose(fp);
-	  return FALSE;
-	}
-	++buffpoint;
-      }
-      ++buffpoint;
-      while (*buffpoint != '.')
-      {
-	if ( (*buffpoint == '#') || (*buffpoint == '\0') )
-	{
-	  fprintf(stderr, "\n%s: WARNING: error in direct command file - missing , \n",progname);
-	  fclose(fp);
-	  return FALSE;
-	}
-	if (*buffpoint == '*')
+      if (buffpoint[0] == '*')
 	{
 	  ++buffpoint;
-	  if (WriteFontName(&buffpoint, fRtf)==FALSE)
-	  {
-	    fprintf(stderr, "\n%s: WARNING: error in direct command file - invalid font name , \n",progname);
-	    fclose(fp);
-	    return FALSE;
-	  }
+	  (void)WriteFontName(&buffpoint, fRtf);
+
+	  /* From here on it is not necesarry
+	     if (WriteFontName(&buffpoint, fRtf))
+	     {
+	     fprintf(stderr,
+	     "\n%s: WARNING: error in direct command file"
+	     " - invalid font name , \n",
+	     progname);
+	     return FALSE;
+	     }
+	     */
 	}
-	else
+      else
+	{
 	  fprintf(fRtf,"%c",*buffpoint);
-	++buffpoint;
-      }
-      fclose(fp);
-      return TRUE;
-    } /* end found */
-  }  /* end while */
+	}
+      
+      ++buffpoint;
+      
+    }  /* end while */
+  return TRUE;
 }
+
+
